@@ -28,7 +28,7 @@ def autogen_C_logging_code(logging_dict, verbose=0):
                 'model.dt'
             ]
         }
-        
+
     """
 
     # Check if the input is of the correct form
@@ -256,13 +256,15 @@ def compile_shared_lib(code, sourcefile_name, outfile_name, verbose=0):
     compilation_dict = return_compilation_dict(verbose)
 
     # Construct full command
-    command = "{cc} -fPIC {ccflags} {libs} -o {outfile_name} {sourcefile_name} {inc}".format(
-        cc=compilation_dict["cc"],
-        ccflags=compilation_dict["ccflags"],
-        libs=compilation_dict["libs"],
-        outfile_name=outfile_name,
-        sourcefile_name=sourcefile_name,
-        inc=compilation_dict["inc"],
+    command = (
+        "{cc} -fPIC {ccflags} {libs} -o {outfile_name} {sourcefile_name} {inc}".format(
+            cc=compilation_dict["cc"],
+            ccflags=compilation_dict["ccflags"],
+            libs=compilation_dict["libs"],
+            outfile_name=outfile_name,
+            sourcefile_name=sourcefile_name,
+            inc=compilation_dict["inc"],
+        )
     )
 
     # remove extra whitespaces:
@@ -320,7 +322,8 @@ def create_and_load_logging_function(custom_logging_code, verbose=0):
     _ = ctypes.CDLL("libbinary_c.so", mode=ctypes.RTLD_GLOBAL)
 
     libcustom_logging = ctypes.CDLL(
-        library_name, mode=ctypes.RTLD_GLOBAL,
+        library_name,
+        mode=ctypes.RTLD_GLOBAL,
     )  # loads the shared library
 
     # Get memory adress of function. mimicking a pointer
